<?php
/**
 * General Admin functionality - hooks, methods.
 *  
 * This file serves to be the functions.php for admin functionality. Any
 * non-specific functionality is contained here.
 * 
 * Also see admin/ folder in the root.
 *
 */
class Bunyad_Theme_Admin
{
	public function __construct()
	{
		// Setup plugins before init
		$this->setup_plugins();

		// Set up hooks
		add_action('bunyad_theme_init', [$this, 'init']);

		/**
		 * Include relevant admin files
		 */
		
		// Dashboard, importer and editor
		include get_template_directory() . '/inc/admin/dashboard.php';
		include get_template_directory() . '/inc/admin/import.php';
		include get_template_directory() . '/inc/admin/editor.php';

		// Admin notices.
		Bunyad::register('admin_notices', [
			'class' => '\Bunyad\Core\AdminNotices\Module',
			'init'  => true
		]);

		// Migrations / updates.
		require_once get_theme_file_path('inc/admin/migrations.php');

		// Packaged plugins updates: Only if Sphere Core is inactive or older than 1.4.
		if (
			!class_exists('\Sphere\Core\Plugin', false) 
			|| version_compare(\Sphere\Core\Plugin::VERSION, '1.4.0', '<')
		) {
			include get_template_directory() . '/inc/admin/plugins-update.php';
		}

		// Since 5.8+, block widgets can cause issues.
		add_action('after_switch_theme', [$this, 'disable_unsupported_widgets']);
	}
	
	public function init()
	{
		// Do 3rd party plugin settings, if needed, at theme init.
		add_action('after_switch_theme', [$this, 'plugin_settings']);

		// Educate on theme check
		add_action('themecheck_checks_loaded', array($this, 'notice_themecheck'));
		
		// Compatability warning
		if (class_exists('Endurance_Page_Cache')) {
			add_action('admin_notices', array($this, 'notice_cache'));
		}
		
	}

	/**
	 * Setup and recommend plugins.
	 */
	public function setup_plugins()
	{
		if (!is_admin()) {
			return;
		}

		$plugins = include get_template_directory() . '/inc/admin/theme-plugins.php';

		// Set for update checking.
		Bunyad::registry()->set('packaged_plugins', $plugins);
		
		// Only load and register if in admin (checked above), or if user has permissions.
		if (current_user_can('install_plugins')) {
			// Load the plugin activation class and our enhancements.
			require_once get_template_directory() . '/lib/vendor/class-tgm-plugin-activation.php';
			require_once get_template_directory() . '/inc/admin/dash-plugins.php';

			tgmpa($plugins, [
				'parent_slug' => 'sphere-dash',
				'id'          => 'cheerup_tgmpa'
			]);
		}
	}
	
	/**
	 * Educate new users about theme check 
	 */
	public function notice_themecheck() 
	{
		if (!isset($_GET['page']) OR $_GET['page'] != 'themecheck') {
			return;
		}
		?>
		
		<div class="error">
			<h3>Theme Check Invalid for Premium Themes!</h3>
			<p>
			Theme Check plugin was created for WordPress.org repository to automate submission checks. Please note that theme check rules DO NOT apply to premium themes. 
			</p>
		</div>
		<?php 
	}
	
	
	/**
	 * Compatibility Issue notice
	 * 
	 * - BlueHost Endurance Cache plugin
	 */
	public function notice_cache()
	{
		$cache = get_option('mm_cache_settings');
		if (!empty($cache['page']) && $cache['page'] == 'disabled') {
			return;
		}

		// The level is probably 0.
		if (!get_option('endurance_cache_level')) {
			return;
		}
		
		if (!empty($_GET['page']) && strstr($_GET['page'], 'sphere-')) {
			return;
		}
		?>
		
		<div class="error">
			<p>
			<strong>Incompatible Plugin:</strong> Endurance Cache Plugin does not follow best practices for cache and is not recommended. 
				Please disable it now. Go to <a href="<?php echo esc_url(admin_url('options-general.php#epc_settings')); ?>">Settings > General</a>, set Cache Level to Off and Save.
				After your site is setup, consider a better cache plugin like W3 Total Cache  (<a href="https://theme-sphere.com/docs/cheerup/#performance" target="_blank">more info</a>).
			</p> 
		</div>
		<?php 
	}

	/**
	 * Deactivate unsupported block widgets.
	 */
	public function disable_unsupported_widgets()
	{
		// Not a first install. Widgets were previously set. Don't have to do anything.
		if (get_theme_mod('sidebars_widgets')) {
			return;
		}

		$sidebar_widgets = get_option('sidebars_widgets', []);
		if (empty($sidebar_widgets)) {
			return;
		}

		foreach ($sidebar_widgets as $sidebar => $widgets) {
			foreach ($widgets as $key => $id) {

				// Move to inactive.
				if (substr($id, 0, 6) === 'block-') {
					$sidebar_widgets['wp_inactive_widgets'][] = $id;
					unset($sidebar_widgets[$sidebar][$key]);
				}
			}
		}

		update_option('sidebars_widgets', $sidebar_widgets);
	}

	/**
	 * 3rd Party Settings/Options update on theme activate.
	 */
	public function plugin_settings()
	{
		$this->do_wprm_settings();
	}

	/**
	 * WP Recipe Maker initial settings.
	 */
	public function do_wprm_settings()
	{
		$options = (array) get_option('wprm_settings');
		$options['default_recipe_template_modern'] = 'cheerup';
		$options['comment_rating_form_position']   = 'below';
		$options['default_recipe_amp_template']    = 'default_recipe_template';
		$options['default_recipe_feed_template']   = 'default_recipe_template';

		update_option('wprm_settings', $options);
	}
}

// init and make available in Bunyad::get('admin')
Bunyad::register('admin', array(
	'class' => 'Bunyad_Theme_Admin',
	'init' => true
));